#ifndef PAGING_H
#define PAGING_H

#include "os.h"
#include "ordered_array.h"

// Macros used in the bitset algorithms.
#define INDEX_FROM_BIT(a)   (a/(8*4))
#define OFFSET_FROM_BIT(a)  (a%(8*4))

ULONG*  frames;
ULONG  nframes;

typedef struct page
{
    ULONG present    :  1;   // Page present in memory
    ULONG rw         :  1;   // Read-only if clear, readwrite if set
    ULONG user       :  1;   // Supervisor level only if clear
    ULONG accessed   :  1;   // Has the page been accessed since last refresh?
    ULONG dirty      :  1;   // Has the page been written to since last refresh?
    ULONG unused     :  7;   // Amalgamation of unused and reserved bits
    ULONG frame      : 20;   // Frame address (shifted right 12 bits)
} page_t;

typedef struct page_table
{
    page_t pages[1024];
} page_table_t;

typedef struct page_directory
{
    page_table_t* tables[1024];
    ULONG tablesPhysical[1024];
    ULONG physicalAddr;
} page_directory_t;

void              paging_install();
void              switch_page_directory(page_directory_t* new);
page_t*           get_page(ULONG address, int make, page_directory_t* dir);
void              page_fault(registers_t* regs);
page_directory_t* clone_directory(page_directory_t* src);

void              alloc_frame(page_t *page, int is_kernel, int is_writeable);
void              free_frame(page_t *page);

extern void copy_page_physical(ULONG source_address, ULONG dest_address);

#endif
